@@ -6,41 +6,58 @@ import dbt.exceptions
 from dbt.adapters.base import Credentials
 from dbt.adapters.sql import SQLConnectionManager
 from dbt.contracts.connection import AdapterResponse
-from dbt.logger import GLOBAL_LOGGER as logger
+from dbt.events import AdapterLogger
 
 from dbt.helper_types import Port
 from dataclasses import dataclass
 from typing import Optional
 
 
+logger = AdapterLogger("Postgres")
+
+
 @dataclass
 class PostgresCredentials(Credentials):
     host: str
     user: str
     port: Port
     password: str  # on postgres the password is mandatory
+    connect_timeout: int = 10
     role: Optional[str] = None
     search_path: Optional[str] = None
     keepalives_idle: int = 0  # 0 means to use the default value
     sslmode: Optional[str] = None
-    application_name: Optional[str] = 'dbt'
+    sslcert: Optional[str] = None
+    sslkey: Optional[str] = None
+    sslrootcert: Optional[str] = None
+    application_name: Optional[str] = "dbt"
+    retries: int = 1
 
-    _ALIASES = {
-        'dbname': 'database',
-        'pass': 'password'
-    }
+    _ALIASES = {"dbname": "database", "pass": "password"}
 
     @property
     def type(self):
-        return 'postgres'
+        return "postgres"
+
+    @property
+    def unique_field(self):
+        return self.host
 
     def _connection_keys(self):
-        return ('host', 'port', 'user', 'database', 'schema', 'search_path',
-                'keepalives_idle', 'sslmode')
+        return (
+            "host",
+            "port",
+            "user",
+            "database",
+            "schema",
+            "search_path",
+            "keepalives_idle",
+            "sslmode",
+        )
 
 
 class PostgresConnectionManager(SQLConnectionManager):
-    TYPE = 'postgres'
+    TYPE = "postgres"
 
     @contextmanager
     def exception_handler(self, sql):
@@ -48,7 +65,7 @@ class PostgresConnectionManager(SQLConnectionManager):
             yield
 
         except psycopg2.DatabaseError as e:
-            logger.debug('Postgres error: {}'.format(str(e)))
+            logger.debug("Postgres error: {}".format(str(e)))
 
             try:
                 self.rollback_if_open()
@@ -72,8 +89,8 @@ class PostgresConnectionManager(SQLConnectionManager):
 
     @classmethod
     def open(cls, connection):
-        if connection.state == 'open':
-            logger.debug('Connection is already open, skipping open.')
+        if connection.state == "open":
+            logger.debug("Connection is already open, skipping open.")
             return connection
 
         credentials = cls.get_credentials(connection.credentials)
@@ -81,48 +98,60 @@ class PostgresConnectionManager(SQLConnectionManager):
         # we don't want to pass 0 along to connect() as postgres will try to
         # call an invalid setsockopt() call (contrary to the docs).
         if credentials.keepalives_idle:
-            kwargs['keepalives_idle'] = credentials.keepalives_idle
+            kwargs["keepalives_idle"] = credentials.keepalives_idle
 
         # psycopg2 doesn't support search_path officially,
         # see https://github.com/psycopg/psycopg2/issues/465
         search_path = credentials.search_path
-        if search_path is not None and search_path != '':
+        if search_path is not None and search_path != "":
             # see https://postgresql.org/docs/9.5/libpq-connect.html
-            kwargs['options'] = '-c search_path={}'.format(
-                search_path.replace(' ', '\\ '))
+            kwargs["options"] = "-c search_path={}".format(search_path.replace(" ", "\\ "))
 
         if credentials.sslmode:
-            kwargs['sslmode'] = credentials.sslmode
+            kwargs["sslmode"] = credentials.sslmode
+
+        if credentials.sslcert is not None:
+            kwargs["sslcert"] = credentials.sslcert
+
+        if credentials.sslkey is not None:
+            kwargs["sslkey"] = credentials.sslkey
+
+        if credentials.sslrootcert is not None:
+            kwargs["sslrootcert"] = credentials.sslrootcert
 
         if credentials.application_name:
-            kwargs['application_name'] = credentials.application_name
+            kwargs["application_name"] = credentials.application_name
 
-        try:
+        def connect():
             handle = psycopg2.connect(
                 dbname=credentials.database,
                 user=credentials.user,
                 host=credentials.host,
                 password=credentials.password,
                 port=credentials.port,
-                connect_timeout=10,
-                **kwargs)
-
+                connect_timeout=credentials.connect_timeout,
+                **kwargs,
+            )
             if credentials.role:
-                handle.cursor().execute('set role {}'.format(credentials.role))
+                handle.cursor().execute("set role {}".format(credentials.role))
+            return handle
 
-            connection.handle = handle
-            connection.state = 'open'
-        except psycopg2.Error as e:
-            logger.debug("Got an error when attempting to open a postgres "
-                         "connection: '{}'"
-                         .format(e))
+        retryable_exceptions = [
+            # OperationalError is subclassed by all psycopg2 Connection Exceptions and it's raised
+            # by generic connection timeouts without an error code. This is a limitation of
+            # psycopg2 which doesn't provide subclasses for errors without a SQLSTATE error code.
+            # The limitation has been known for a while and there are no efforts to tackle it.
+            # See: https://github.com/psycopg/psycopg2/issues/682
+            psycopg2.errors.OperationalError,
+        ]
 
-            connection.handle = None
-            connection.state = 'fail'
-
-            raise dbt.exceptions.FailedToConnectException(str(e))
-
-        return connection
+        return cls.retry_connection(
+            connection,
+            connect=connect,
+            logger=logger,
+            retry_limit=credentials.retries,
+            retryable_exceptions=retryable_exceptions,
+        )
 
     def cancel(self, connection):
         connection_name = connection.name
@@ -130,10 +159,8 @@ class PostgresConnectionManager(SQLConnectionManager):
             pid = connection.handle.get_backend_pid()
         except psycopg2.InterfaceError as exc:
             # if the connection is already closed, not much to cancel!
-            if 'already closed' in str(exc):
-                logger.debug(
-                    f'Connection {connection_name} was already closed'
-                )
+            if "already closed" in str(exc):
+                logger.debug(f"Connection {connection_name} was already closed")
                 return
             # probably bad, re-raise it
             raise
@@ -156,14 +183,6 @@ class PostgresConnectionManager(SQLConnectionManager):
         message = str(cursor.statusmessage)
         rows = cursor.rowcount
         status_message_parts = message.split() if message is not None else []
-        status_messsage_strings = [
-            part
-            for part in status_message_parts
-            if not part.isdigit()
-        ]
-        code = ' '.join(status_messsage_strings)
-        return AdapterResponse(
-            _message=message,
-            code=code,
-            rows_affected=rows
-        )
+        status_messsage_strings = [part for part in status_message_parts if not part.isdigit()]
+        code = " ".join(status_messsage_strings)
+        return AdapterResponse(_message=message, code=code, rows_affected=rows)
